import { useEffect, useState } from "react";
import { IntlProvider } from "react-intl";
import type { WithChildren, WithClassName } from "../../utils/types";
import Logger from "../../Logger";
import { useLanguageStore } from "../../data/state/language";
import LoadingIndicator from "../widgets/LoadingIndicator";
import Events from "../../data/events/Events";
import { EventTypes } from "../../data/events/eventTypes";
import useEvents from "../../data/events/useEvents";
import Overlay from "../widgets/Overlay";
import classNames from "classnames";
import { STYLES } from "../../constants";

const TAG = "AppIntlProvider";

export type IntlMessageMap = Record<string, string>;

export default function AppIntlProvider({ children }: WithChildren) {
	const language = useLanguageStore(s => s.language);
	const [messages, setMessages] = useState<IntlMessageMap | null>(null);
	const [isLoading, setLoading] = useState<boolean>(false);
	const [error, setError] = useState<Error | null>(null);

	useEvents(data => {
		switch (data.type) {
			case EventTypes.LANGUAGE_LOAD_FAIL:
				// Can't really work without strings, so hard-crash here to give
				// the user a semi-good error message using the error boundary.
				data.error.message = `Failed to load app language file: ${data.error.message}`;
				setError(data.error);
				break;
			case EventTypes.LANGUAGE_LOAD_SUCCESS:
				setMessages(data.language);
				setLoading(false);
				break;
		}
	});

	useEffect(() => {
		loadLanguage(language);
		setLoading(true);
	}, [language]);

	if (error) {
		throw error;
	}

	if (!messages) {
		return <Loading className={`mt-16 ${STYLES.TEXT_COLOR}`} />;
	}

	return (
		<IntlProvider locale={language} messages={messages}>
			{children}
			<Overlay isVisible={isLoading}>
				<Loading />
			</Overlay>
		</IntlProvider>
	);
}

export function preloadActiveLanguage() {
	void loadLanguageFile(useLanguageStore.getState().language);
}

function Loading({ className }: WithClassName) {
	return <LoadingIndicator className={classNames("font-bold text-2xl", className)}>Loading App…</LoadingIndicator>;
}

function loadLanguage(lang: string) {
	loadLanguageFile(lang)
		.then(language => Events.emit(EventTypes.LANGUAGE_LOAD_SUCCESS, { language }))
		.catch(error => {
			Events.emit(EventTypes.LANGUAGE_LOAD_FAIL, { error: error as Error });
			Logger.error(TAG, `Error loading language ${lang}`, error);
		});
}

async function loadLanguageFile(lang: String): Promise<IntlMessageMap> {
	switch (lang) {
		case "en":
			return (await import("./lang/en.json")).default;
		case "de":
			return (await import("./lang/de.json")).default;
		default:
			throw new Error(`Attemping to load invalid language '${lang}'`);
	}
}
