import classNames from "classnames";
import Spinner from "./Spinner";

type ButtonStyle = "info" | "success" | "warn" | "danger" | "link";

type ButtonOrLinkProps =
	| (React.ButtonHTMLAttributes<HTMLButtonElement> & { as?: "button" })
	| (React.AnchorHTMLAttributes<HTMLAnchorElement> & { as: "a" });

type Props = ButtonOrLinkProps & {
	buttonStyle?: ButtonStyle;
	small?: boolean;
	outlined?: boolean;
	loading?: boolean;
};

const buttonStyleToClass: Record<ButtonStyle, string> = {
	info: "text-white bg-blue-500 hover:bg-blue-600 dark:bg-blue-800 dark:hover:bg-blue-900",
	success: "text-white bg-green-600 hover:bg-green-700 dark:bg-green-800 dark:hover:bg-green-900",
	warn: "text-black bg-yellow-500 hover:bg-yellow-600 dark:bg-yellow-600 dark:hover:bg-yellow-700",
	danger: "text-white bg-red-500 hover:bg-red-600 dark:bg-red-800 dark:hover:bg-red-900",
	link: "text-inherit hover:opacity-75 hover:underline",
};

const buttonStyleToClassOutlined: Record<ButtonStyle, string> = {
	info: "border-blue-500 hover:bg-blue-500 dark:border-blue-800 dark:hover:bg-blue-800 hover:text-white",
	success: "border-green-600 hover:bg-green-600 dark:border-green-800 dark:hover:bg-green-800 hover:text-white",
	warn: "border-yellow-500 hover:bg-yellow-500 dark:border-yellow-600 dark:hover:bg-yellow-600 hover:text-black",
	danger: "border-red-500 hover:bg-red-500 dark:border-red-800 dark:hover:bg-red-800 hover:text-white",
	link: "border-slate-300 text-inherit hover:opacity-75 hover:underline",
};

const buttonStyleToClassDisabled: Record<ButtonStyle, string> = {
	info: "bg-blue-500/25 dark:bg-blue-800/50",
	success: "bg-green-500/25 dark:bg-green-800/50",
	warn: "bg-yellow-500/25 dark:bg-yellow-800/50",
	danger: "bg-red-500/25 dark:bg-red-800/50",
	link: "",
};

export default function Button({ buttonStyle, small, outlined, loading, className, children, ...props }: Props) {
	const disabled = "disabled" in props && props.disabled;
	if (loading) {
		outlined = false;
	}
	if (disabled) {
		outlined = false;
	}

	const classes = classNames(
		"leading-none rounded flex flex-row items-center justify-center gap-2 transition",
		!disabled && !outlined && buttonStyleToClass[buttonStyle ?? "info"],
		!disabled && outlined && buttonStyleToClassOutlined[buttonStyle ?? "info"],
		disabled && buttonStyleToClassDisabled[buttonStyle ?? "info"],
		{
			["text-inherit border"]: outlined,
			// Force the same spacing as outlined buttons by creating a fake border
			["border border-transparent"]: !outlined,
			["text-gray-500 dark:text-gray-400 cursor-not-allowed"]: disabled,
			["px-4 py-4"]: !small,
			["px-2.5 py-3"]: small,
		},
		className
	);

	if (!("href" in props)) {
		props.type ??= "button";
	}

	const commonProps = {
		className: classes,
		disabled: disabled || loading,
	};

	const commonChildren = (
		<>
			{loading && <Spinner className="w-4 h-4 fill-current" />}
			{children}
		</>
	);

	// Element creation is separated like this for type safety
	if (props.as === "a") {
		return (
			<a {...props} {...commonProps}>
				{commonChildren}
			</a>
		);
	}
	return (
		<button {...props} {...commonProps}>
			{commonChildren}
		</button>
	);
}
