import { useState } from "react";
import ApiActions from "../../../data/ApiActions";
import { useDiscordStore } from "../../../data/state/discord";
import Button from "../Button";
import useEvents from "../../../data/events/useEvents";
import { EventTypes } from "../../../data/events/eventTypes";
import P from "../typography/Paragraph";
import { OAuthType, startOAuthFlow } from "../../../utils/oauth";
import Message from "../../i18n/Message";
import { STYLES } from "../../../constants";
import type { WithClassName } from "../../../utils/types";
import classNames from "classnames";

export default function DiscordConnectionManagement({ className }: WithClassName) {
	const user = useDiscordStore(s => s.data?.user);
	const [isShowingDisconnectionWarning, setShowDisconnectionWarning] = useState<boolean>(false);
	const [isLoading, setLoading] = useState<boolean>(false);
	const [errorMessage, setErrorMessage] = useState<string | null>(null);

	useEvents(data => {
		switch (data.type) {
			case EventTypes.DISCORD_EXCHANGE_CODE_SUCCESS:
			case EventTypes.REMOVE_DISCORD_CONNECTION_SUCCESS:
				setShowDisconnectionWarning(false);
				setLoading(false);
				setErrorMessage(null);
				break;
			case EventTypes.DISCORD_EXCHANGE_CODE_FAIL:
			case EventTypes.REMOVE_DISCORD_CONNECTION_FAIL:
			case EventTypes.POPUP_FAIL:
				setLoading(false);
				setErrorMessage(data.error.message);
				break;
		}
	});

	const onConnectionButtonClick = () => {
		if (isLoading) {
			return;
		}
		if (user) {
			// Discord user exists, prompt for disconnection
			setShowDisconnectionWarning(true);
		} else {
			// No Discord user, initiate new connection
			setLoading(true);
			setErrorMessage(null);
			startOAuthFlow(OAuthType.DISCORD);
		}
	};

	const onDisconnectConfirmClick = () => {
		setLoading(true);
		setErrorMessage(null);
		ApiActions.removeDiscordConnection();
	};

	const onDisconnectCancelClick = () => {
		setShowDisconnectionWarning(false);
		setErrorMessage(null);
	};

	const renderedErrorMessage = errorMessage && (
		<div className={`${STYLES.ERROR_COLOR}`}>
			<Message id="discord.connection.error" vars={{ errorMessage }} />
		</div>
	);

	const innerContents = isShowingDisconnectionWarning ? (
		<div className="flex flex-col">
			<P className="text-center whitespace-pre-line">
				<Message
					id="discord.connection.unlink-account.warning"
					vars={{
						username: user!.username,
					}}
				/>
			</P>
			<div className="flex flex-row gap-4 justify-center">
				<Button buttonStyle="success" small disabled={isLoading} onClick={onDisconnectCancelClick}>
					<Message id="discord.connection.unlink-account.cancel" />
				</Button>
				<Button buttonStyle="danger" small outlined loading={isLoading} onClick={onDisconnectConfirmClick}>
					<Message id="discord.connection.unlink-account.confirm" />
				</Button>
			</div>
		</div>
	) : (
		<div className="flex flex-col md:flex-row gap-4 justify-between items-center">
			<div>
				{user ? (
					<Message
						id="discord.connection.is-linked-with"
						vars={{
							username: user.username,
							userId: user.id,
						}}
					/>
				) : (
					<Message id="discord.connection.is-not-linked" />
				)}
			</div>
			<Button
				onClick={onConnectionButtonClick}
				buttonStyle={user ? "danger" : "success"}
				outlined={!!user}
				small={!!user}
				loading={isLoading}>
				{user ? <Message id="discord.connection.unlink-account" /> : <Message id="discord.connection.link-account" />}
			</Button>
		</div>
	);

	return (
		<div className={classNames("p-4 border rounded-lg flex flex-col gap-4", STYLES.BORDER_COLOR, className)}>
			{innerContents}
			{renderedErrorMessage}
		</div>
	);
}
