import { Fragment } from "react";
import { DISCORD_GUILD_ICON_URL, DISCORD_INVITE_URL, STYLES } from "../../../constants";
import { colorToHex } from "../../../utils/utils";
import Message from "../../i18n/Message";
import Button from "../Button";
import fallbackDiscordIcon from "./generic-disccord-icon.png";

export type DiscordServerListServerProps = GuildProps;

interface Props {
	title: string;
	emptyPlaceholder: string;
	guilds: GuildProps[];
}

export default function DiscordServerList({ title, emptyPlaceholder, guilds }: Props) {
	const classes =
		// Grid Common
		"grid gap-4 p-4 " +
		// Grid Small
		"grid-cols-1 max-md:justify-items-center " +
		// Grid Large
		"md:grid-cols-[auto_1fr_1fr_auto_auto] items-center " +
		// Colors
		`border ${STYLES.BORDER_COLOR} ` +
		// Misc
		"rounded-lg overflow-hidden leading-none";

	const headerClasses =
		"col-span-full justify-self-stretch border-b font-bold text-lg -mx-4 -mt-4 px-4 py-2 " +
		`bg-slate-50 dark:bg-slate-700 ${STYLES.BORDER_COLOR}`;

	return (
		<div className={classes}>
			<h3 className={headerClasses}>{title}</h3>
			{guilds.length ? (
				<div className="contents max-md:hidden text-slate-600 dark:text-slate-400">
					<div className="col-span-2">
						<Message id="widgets.discord.server" />
					</div>
					<div className="col-span-3">
						<Message id="widgets.discord.roles" />
					</div>
					<div className={`col-span-full border-t ${STYLES.BORDER_COLOR} -mx-4`} />
				</div>
			) : (
				<div className={`col-span-full text-center ${STYLES.MUTED_COLOR} whitespace-pre-line ${STYLES.LINE_HEIGHT}`}>
					{emptyPlaceholder}
				</div>
			)}
			{guilds.map(g => (
				<Fragment key={g.id}>
					<Guild {...g} />
					<div className={`last:hidden col-span-full justify-self-stretch border-t ${STYLES.BORDER_COLOR} -mx-4`} />
				</Fragment>
			))}
		</div>
	);
}

interface GuildProps {
	id: string;
	name: string;
	iconHash: string | null;
	memberCount: number;
	inviteCode: string;
	roles: RolePillProps[];
	isMember: boolean;
}

function Guild({ id, name, iconHash, memberCount, inviteCode, roles, isMember }: GuildProps) {
	const guildIconUrl = iconHash ? DISCORD_GUILD_ICON_URL(id, iconHash) : fallbackDiscordIcon;
	return (
		<>
			<img
				className={`w-14 h-14 md:w-12 md:h-12 object-contain rounded-xl border ${STYLES.BORDER_COLOR} max-md:mt-4`}
				src={guildIconUrl}
			/>
			<div className="truncate">{name}</div>
			<ul className="flex flex-row flex-wrap items-center gap-2">
				{roles.map(r => (
					<RolePill key={r.id} {...r} />
				))}
			</ul>
			<div className="flex flex-row gap-2 items-center">
				<div className="w-2 h-2 bg-green-600 rounded-full" />
				<Message
					id="widgets.discord.members"
					vars={{
						members: memberCount,
					}}
				/>
			</div>
			<div className="max-md:mb-4">
				{isMember ? (
					<Button small disabled>
						<Message id="widgets.discord.joined" />
					</Button>
				) : (
					<Button as="a" small href={DISCORD_INVITE_URL(inviteCode)}>
						<Message id="widgets.discord.join" />
					</Button>
				)}
			</div>
		</>
	);
}

interface RolePillProps {
	id: string;
	name: string;
	color: number;
}

function RolePill({ name, color }: RolePillProps) {
	const colorString = colorToHex(color);
	return (
		<li
			className="flex flex-row items-baseline gap-1 border rounded-full pl-1.5 pr-2 py-1"
			style={{
				borderColor: colorString,
			}}
			aria-label={`Discord Role: ${name}`}>
			<div
				className="w-3 h-3 rounded-full"
				style={{
					backgroundColor: colorString,
				}}
			/>
			<div className="truncate max-w-[12rem]">{name}</div>
		</li>
	);
}
