import Logger from "../Logger";
import type { ApiResponse } from "./ApiRequests";
import ApiRequests from "./ApiRequests";
import Events from "./events/Events";
import { EventTypes } from "./events/eventTypes";
import type { ApiGetDiscordResponse, ApiGetMeResponse, ApiPatchMeRequest } from "./schemas";
import { useDiscordStore } from "./state/discord";
import { useLoginStore } from "./state/login";
import { useUserStore } from "./state/user";

const mockUser = {
	name: "Adrian Paschkowski",
	email: "adrian.paschkowski@ruhr-uni-bochum.de",
	course: "Angewandte Informatik",
	groups: [
		"/Angewandte Informatik",
		"/Angewandte Informatik/FSR-AI",
		"/Angewandte Informatik/FSR-AI/FSR-AI Serveradmins",
	],
} satisfies ApiGetMeResponse;

// eslint-disable-next-line @typescript-eslint/no-unused-vars
const mockDiscordUser = {
	id: "138036023266508800",
	username: "fe80",
};

const mockDiscordConnection = {
	user: mockDiscordUser,
	guilds: [
		{
			id: "167288845245743104",
			name: "[AI][AI][AI]!",
			icon_hash: "1e3cac2ebd982d12310feec089354037",
			member_count: 866,
			invite_code: "U83jb9k",
			roles: [
				{
					id: "385164241256185856",
					name: "FS",
					color: 0x03bfc4,
				},
				{
					id: "372778212666769409",
					name: "FSR",
					color: 0x0077ff,
				},
			],
			is_member: true,
		},
		{
			id: "873174739449958430",
			name: "Fachschaft Informatik",
			icon_hash: "f958829dd798be4bf77f6bc51d4102ad",
			member_count: 414,
			invite_code: "",
			roles: [
				{
					id: "1027749947816288267",
					name: "AI",
					color: 0x2ecc71,
				},
			],
			is_member: false,
		},
		{
			id: "699314835543949383",
			name: "Fachschaft IT-Sicherheit",
			icon_hash: "e3ee2db921213b178f77bac5dfb80fec",
			member_count: 1170,
			invite_code: "DcWEU9w93P",
			roles: [
				{
					id: "1052253043158171719",
					name: "Angewandte Informatik",
					color: 0x1abc9c,
				},
			],
			is_member: false,
		},
	],
} satisfies ApiGetDiscordResponse;

const TAG = "ApiActions";

export default class ApiActions {
	public static fetchSelf(): void {
		Events.emit(EventTypes.FETCH_SELF);
		this.invokeApi(
			async () => {
				// TODO This is mock data
				await new Promise(r => setTimeout(r, 400));
				return {
					status: 200,
					ok: true,
					error: false,
					data: mockUser,
				};
				// ApiRequests.getMe()
			},
			user => {
				useUserStore.getState().setData(user);
				Events.emit(EventTypes.FETCH_SELF_SUCCESS, { user });
			},
			error => Events.emit(EventTypes.FETCH_SELF_FAIL, { error })
		);
	}

	public static updateSelf(data: ApiPatchMeRequest): void {
		Events.emit(EventTypes.UPDATE_SELF);
		this.invokeApi(
			() => ApiRequests.patchMe(data),
			user => {
				useUserStore.getState().setData(user);
				Events.emit(EventTypes.UPDATE_SELF_SUCCESS, { user });
				if (data.course) {
					// Updating the own study course affects what discord roles you are eligible for.
					// We may have stale data in cache, so let's fetch the most recent state.
					this.fetchDiscordConnection();
				}
			},
			error => Events.emit(EventTypes.UPDATE_SELF_FAIL, { error })
		);
	}

	public static fetchDiscordConnection(): void {
		Events.emit(EventTypes.FETCH_DISCORD_CONNECTION);
		this.invokeApi(
			async () => {
				// TODO This is mock data
				await new Promise(r => setTimeout(r, 400));
				return {
					status: 200,
					ok: true,
					error: false,
					data: mockDiscordConnection,
				};
				// ApiRequests.getDiscord()
			},
			discord => {
				useDiscordStore.getState().setData(discord);
				Events.emit(EventTypes.FETCH_DISCORD_CONNECTION_SUCCESS, { discord });
			},
			error => Events.emit(EventTypes.FETCH_DISCORD_CONNECTION_FAIL, { error })
		);
	}

	public static removeDiscordConnection(): void {
		Events.emit(EventTypes.REMOVE_DISCORD_CONNECTION);
		this.invokeApi(
			() => ApiRequests.deleteDiscord(),
			discord => {
				useDiscordStore.getState().setData(discord);
				Events.emit(EventTypes.REMOVE_DISCORD_CONNECTION_SUCCESS, { discord });
			},
			error => Events.emit(EventTypes.REMOVE_DISCORD_CONNECTION_FAIL, { error })
		);
	}

	public static exchangeDiscordCode(code: string): void {
		Events.emit(EventTypes.DISCORD_EXCHANGE_CODE);
		this.invokeApi(
			() => ApiRequests.postAuthDiscordToken({ code }),
			discord => {
				useDiscordStore.getState().setData(discord);
				Events.emit(EventTypes.DISCORD_EXCHANGE_CODE_SUCCESS, { discord });
			},
			error => Events.emit(EventTypes.DISCORD_EXCHANGE_CODE_FAIL, { error })
		);
	}

	public static exchangeRubCode(code: string): void {
		Events.emit(EventTypes.RUB_EXCHANGE_CODE);
		this.invokeApi(
			async () => {
				// TODO This is mock data
				await new Promise(r => setTimeout(r, 400));
				return {
					status: 200,
					ok: true,
					error: false,
					data: {
						authorization: "beans",
					},
				};
				// ApiRequests.postAuthRubToken({ code })
			},
			data => {
				useLoginStore.getState().setToken(data.authorization);
				Events.emit(EventTypes.RUB_EXCHANGE_CODE_SUCCESS);
			},
			error => Events.emit(EventTypes.RUB_EXCHANGE_CODE_FAIL, { error })
		);
	}

	public static logout(): void {
		Events.emit(EventTypes.LOGOUT);
		this.invokeApi(
			() => ApiRequests.postLogout(),
			() => {
				this.clearStores();
				Events.emit(EventTypes.LOGOUT_SUCCESS);
				Events.emit(EventTypes.LOGOUT_COMPLETE);
			},
			error => {
				// Still act as if logout was successful so that the user can't get stuck in a state
				// where the token was actually invalidated but the server still returned an error,
				// and now further logout attempts using an invalid token would fail.
				this.clearStores();
				Events.emit(EventTypes.LOGOUT_FAIL, { error });
				Events.emit(EventTypes.LOGOUT_COMPLETE);
			}
		);
	}

	public static forceLogout(): void {
		this.clearStores();
		Events.emit(EventTypes.LOGOUT_SUCCESS);
		Events.emit(EventTypes.LOGOUT_COMPLETE);
	}

	private static invokeApi<T>(
		start: () => Promise<ApiResponse<T>>,
		success: (data: T) => void,
		error: (error: Error) => void
	): void {
		const run = async () => {
			const res = await start();
			if (res.error) {
				const err = new Error(res.data.error);
				Events.emit(EventTypes.API_CALL_FAIL, {
					error: err,
					response: res,
				});
				error(err);
			} else {
				success(res.data);
			}
		};
		run().catch(e => Logger.error(TAG, "Unexpected error invoking API action", e));
	}

	private static clearStores(): void {
		useLoginStore.getState().setToken(null);
		useDiscordStore.getState().setData(null);
		useUserStore.getState().setData(null);
	}
}
