import type { BaseSchema } from "valibot";
import { safeParse } from "valibot";
import { API_BASE_URL, APP_VERSION } from "../constants";
import {
	ApiGetMeResponseSchema,
	ApiPatchMeResponseSchema,
	ApiGetDiscordResponseSchema,
	ApiDeleteDiscordResponseSchema,
	ApiPostAuthRubResponseSchema,
	ApiPostAuthDiscordTokenResponseSchema,
} from "./schemas";
import type {
	ApiDeleteDiscordResponse,
	ApiGetDiscordResponse,
	ApiGetMeResponse,
	ApiPatchMeRequest,
	ApiPatchMeResponse,
	ApiPostAuthDiscordTokenRequest,
	ApiPostAuthDiscordTokenResponse,
	ApiPostAuthRubResponse,
	ApiPostAuthRubTokenRequest,
} from "./schemas";

import { useLoginStore } from "./state/login";

/* eslint-disable no-mixed-spaces-and-tabs */
export type ApiResponse<T> = {
	status: number;
	ok: boolean;
} & (
	| {
			error: false;
			data: T;
	  }
	| {
			error: true;
			data: ApiErrorResponse;
	  }
);
/* eslint-enable no-mixed-spaces-and-tabs */

export interface ApiErrorResponse {
	error: string;
}

export default class ApiRequests {
	public static async getMe(): Promise<ApiResponse<ApiGetMeResponse>> {
		return this.request("GET", "/me", ApiGetMeResponseSchema);
	}

	public static async patchMe(data: ApiPatchMeRequest): Promise<ApiResponse<ApiPatchMeResponse>> {
		return this.request("PATCH", "/me", ApiPatchMeResponseSchema, data);
	}

	public static async getDiscord(): Promise<ApiResponse<ApiGetDiscordResponse>> {
		return this.request("GET", "/discord", ApiGetDiscordResponseSchema);
	}

	public static async deleteDiscord(): Promise<ApiResponse<ApiDeleteDiscordResponse>> {
		return this.request("DELETE", "/discord", ApiDeleteDiscordResponseSchema);
	}

	public static async postAuthRubToken(data: ApiPostAuthRubTokenRequest): Promise<ApiResponse<ApiPostAuthRubResponse>> {
		return this.request("POST", "/auth/rub/token", ApiPostAuthRubResponseSchema, data);
	}

	public static async postAuthDiscordToken(
		data: ApiPostAuthDiscordTokenRequest
	): Promise<ApiResponse<ApiPostAuthDiscordTokenResponse>> {
		return this.request("POST", "/auth/discord/token", ApiPostAuthDiscordTokenResponseSchema, data);
	}

	public static async postLogout(): Promise<ApiResponse<void>> {
		return this.request("POST", "/logout", null);
	}

	private static get token(): string | null {
		return useLoginStore.getState().token;
	}

	private static async request<T = void>(
		method: string,
		route: String,
		schema: BaseSchema<T> | null,
		body?: unknown
	): Promise<ApiResponse<T>> {
		const url = `${API_BASE_URL}${route}`;
		const headers = new Headers({
			"User-Agent": `FSI Self-Service v${APP_VERSION}`,
			Accept: "application/json",
		});
		if (this.token) {
			headers.set("Authorization", this.token);
		}
		const canHaveBody = method !== "GET" && method !== "HEAD";
		if (canHaveBody && body) {
			headers.set("Content-Type", "application/json");
		}

		let response: Response;
		try {
			response = await fetch(url, {
				method,
				headers,
				body: !canHaveBody || !body ? undefined : JSON.stringify(body),
			});
		} catch (e: unknown) {
			return {
				status: 0,
				ok: false,
				error: true,
				data: {
					error: e instanceof Error ? e.message : "Unknown Network Error",
				},
			};
		}

		const baseResultFields = {
			status: response.status,
			ok: response.ok,
		};

		let json: any = {};
		const text = await response.text();
		if (text && response.headers.get("content-type") === "application/json") {
			try {
				json = JSON.parse(text);
			} catch {
				const limit = 200;
				return {
					...baseResultFields,
					error: true,
					data: {
						error: `Unable to parse response as JSON: ${
							text.length <= limit ? text : `${text.substring(0, limit)}...`
						}`,
					},
				};
			}
		}

		if (!response.ok) {
			return {
				...baseResultFields,
				error: true,
				data: {
					error: `${method} ${route} returned status ${response.status}: ${json.error ?? "Unknown Error"}`,
				},
			};
		}

		if (!schema) {
			return {
				...baseResultFields,
				error: false,
				data: undefined as T,
			};
		}

		const validationResult = safeParse(schema, json);
		if (!validationResult.success) {
			return {
				...baseResultFields,
				error: true,
				data: {
					error: validationResult.issues.map(i => i.message).join(", "),
				},
			};
		}

		return {
			...baseResultFields,
			error: false,
			data: validationResult.output,
		};
	}
}
