import Logger from "../../Logger";
import uniqueId from "../../utils/uniqueId";
import type { EventData, EventTypes } from "./eventTypes";

type EventTypesType = keyof typeof EventTypes;
type EventDataMap = {
	[T in EventTypesType]: T extends keyof EventData ? EventData[T] : {};
};

type ArgfulEventTypesHelper<T extends EventTypesType> = T extends keyof EventData ? T : never;
type ArgfulEventTypes = ArgfulEventTypesHelper<EventTypesType>;
type ArglessEventTypesHelper<T extends EventTypesType> = T extends keyof EventData ? never : T;
type ArglessEventTypes = ArglessEventTypesHelper<EventTypesType>;

type EventCallbackDataMap = {
	[T in EventTypesType]: {
		type: T;
	} & EventDataMap[T];
};

export type EventToken = string;
export type EventCallback = (data: EventCallbackDataMap[EventTypesType]) => void | PromiseLike<void>;

const TAG = "Events";

export default class Events {
	private static listeners: Map<string, EventCallback> = new Map();

	public static on(listener: EventCallback): EventToken {
		const id = uniqueId("evt");
		this.listeners.set(id, listener);
		return id;
	}

	public static off(id: EventToken): void {
		this.listeners.delete(id);
	}

	public static emit(type: ArglessEventTypes): void;
	public static emit<T extends ArgfulEventTypes>(type: T, args: EventDataMap[T]): void;
	public static emit<T extends EventTypesType>(type: T, args?: EventDataMap[T]): void {
		// @ts-expect-error idek
		const data: EventCallbackDataMap[T] = {
			type,
			...args,
		};
		Logger.debug(TAG, `Emitting ${type} to ${this.listeners.size} listeners`);
		for (const callback of this.listeners.values()) {
			let res;
			try {
				// @ts-expect-error see above
				res = callback(data);
				if (!(res instanceof Promise)) {
					continue;
				}
			} catch (e) {
				res = Promise.reject(e);
			}
			res.catch(e => {
				Logger.error(this.name, `Uncaught error emitting event '${type}'`, e);
			});
		}
	}
}
