import Logger from "../Logger";
import ApiActions from "../data/ApiActions";
import Events from "../data/events/Events";
import { EventTypes } from "../data/events/eventTypes";

const TAG = "OAuth";

export enum OAuthType {
	RUB = "rub",
	DISCORD = "discord",
}

const seenCodes = new Set<string>();

export const startOAuthFlow = (type: OAuthType): void => {
	// const url = API_OAUTH_AUTHORIZE_URL(type);
	// TODO Mock data
	const url = "/auth/rub/callback?code=beans";
	const target = `FSI Self-Service ${type} OAuth`;
	const features =
		"popup,menubar=no,location=no,status=no,scrollbars=yes,resizable=yes,width=550,height=850,top=50,left=50";
	let popup = open(url, target, features);
	if (!popup) {
		Logger.warn(TAG, `Failed to open OAuth popup for type '${type}', faling back to normal tab`);
		popup = open(url, target);
	}
	if (!popup) {
		Events.emit(EventTypes.POPUP_FAIL, {
			error: new Error(`Failed to open OAuth popup of type '${type}'. Are you blocking popups?`),
		});
		return;
	}
	popup.focus();
	let awaitingReply = true;

	const healthcheckInterval = setInterval(() => {
		if (popup!.closed && awaitingReply) {
			clearInterval(healthcheckInterval);
			emitError(type, new Error(`Popup closed before exchanging data`));
		}
	}, 500);

	popup.addEventListener("message", e => {
		if (
			e.origin !== location.origin ||
			e.source !== popup ||
			typeof e.data !== "object" ||
			!("messageId" in e.data) ||
			!("type" in e.data) ||
			e.data.type !== `oauth-callback:${type}`
		) {
			return;
		}
		popup!.postMessage(
			{
				messageId: e.data.messageId,
				type: `oauth-confirm:${type}`,
			},
			location.origin
		);
		clearInterval(healthcheckInterval);
		awaitingReply = false;

		const { code, error, errorDescription } = e.data;

		if (typeof error === "string") {
			emitError(
				type,
				new Error(`OAuth Authorization error: ${error}${errorDescription ? `: ${errorDescription}` : ""}`)
			);
			return;
		}

		if (typeof code !== "string") {
			emitError(
				type,
				new Error(
					`Invalid authorization code: Parameter "code" is not of type string, is of type ${
						code === null ? "null" : typeof code
					}`
				)
			);
			return;
		}

		if (seenCodes.has(code)) {
			return;
		}
		// Prevent duplicate requests due to e.g. React dev mode double-executing hooks
		seenCodes.add(code);
		setTimeout(() => seenCodes.clear(), 500);

		switch (type) {
			case OAuthType.RUB:
				ApiActions.exchangeRubCode(code);
				break;
			case OAuthType.DISCORD:
				ApiActions.exchangeDiscordCode(code);
				break;
			default:
				Logger.error(TAG, `Cannot exchange token for unknown OAuth type ${type}`);
		}
	});
};

const emitError = (type: OAuthType, error: Error): void => {
	switch (type) {
		case OAuthType.RUB:
			Events.emit(EventTypes.RUB_EXCHANGE_CODE_FAIL, { error });
			break;
		case OAuthType.DISCORD:
			Events.emit(EventTypes.DISCORD_EXCHANGE_CODE_FAIL, { error });
			break;
		default:
			Logger.error(TAG, `Can't handle error for unknown OAuth type ${type}`);
	}
};
